@@ -1,20 +1,42 @@
-
 package Net::DNS;
-#
-# $Id: DNS.pm 606 2006-09-16 08:03:35Z olaf $
-#
-use strict;
 
-
-BEGIN { 
-    eval { require bytes; }
+#
+# $Id: DNS.pm 981 2012-01-27 23:01:31Z willem $
+#
+use vars qw($SVNVERSION $VERSION);
+BEGIN {
+	$SVNVERSION = (qw$LastChangedRevision: 981 $)[1];
+	$VERSION = '0.68';
 }
 
 
+=head1 NAME
+
+Net::DNS - Perl interface to the Domain Name System
+
+=head1 SYNOPSIS
+
+    use Net::DNS;
+
+=head1 DESCRIPTION
+
+Net::DNS is a collection of Perl modules that act as a Domain
+Name System (DNS) resolver.  It allows the programmer to perform
+DNS queries that are beyond the capabilities of C<gethostbyname>
+and C<gethostbyaddr>.
+
+The programmer should be somewhat familiar with the format of
+a DNS packet and its various sections.  See RFC 1035 or
+I<DNS and BIND> (Albitz & Liu) for details.
+
+=cut
+
+
+
+
 
 use vars qw(
     $HAVE_XS
-    $VERSION
     $DNSSEC
     $DN_EXPAND_ESCAPES
     @ISA
@@ -37,15 +59,30 @@ use vars qw(
 
 
 BEGIN {
-    require DynaLoader;
-    require Exporter;
-    @ISA     = qw(Exporter DynaLoader);
 
-    
-    $VERSION = '0.59';
-    $HAVE_XS = eval { 
+    require Exporter;
+    @ISA     = qw(Exporter );
+    # these need to live here because of dependencies further on.
+    @EXPORT = qw(mx yxrrset nxrrset yxdomain nxdomain rr_add rr_del SEQUENTIAL UNIXTIME YYYYMMDDxx);
+    @EXPORT_OK= qw(name2labels wire2presentation rrsort stripdot);
+
+
+    $HAVE_XS = eval {
 	local $SIG{'__DIE__'} = 'DEFAULT';
-	__PACKAGE__->bootstrap(); 1 
+
+
+	eval {
+		require XSLoader;
+		XSLoader::load('Net::DNS', $VERSION);
+		1;
+	} or do {
+
+		require DynaLoader;
+		push @ISA, 'DynaLoader';
+		bootstrap Net::DNS $VERSION;
+		1;
+	};
+
 	} ? 1 : 0;
 
 }
@@ -54,32 +91,32 @@ BEGIN {
 
 BEGIN {
 
-    $DNSSEC = eval { 
+    $DNSSEC = eval {
 	    local $SIG{'__DIE__'} = 'DEFAULT';
-	    require Net::DNS::SEC; 
-	    1 
+	    require Net::DNS::SEC;
+	    1
 	    } ? 1 : 0;
 
 
 }
 
 
+use strict;
+use Carp;
 use Net::DNS::Resolver;
 use Net::DNS::Packet;
 use Net::DNS::Update;
 use Net::DNS::Header;
 use Net::DNS::Question;
 use Net::DNS::RR;   # use only after $Net::DNS::DNSSEC has been evaluated
-use Carp;
 
-@EXPORT = qw(mx yxrrset nxrrset yxdomain nxdomain rr_add rr_del);
-@EXPORT_OK= qw(name2labels wire2presentation rrsort);
 
 
 #
-# If you implement an RR record make sure you also add it to 
+# If you implement an RR record make sure you also add it to
 # %Net::DNS::RR::RR hash otherwise it will be treated as unknown type.
-# 
+#
+# See http://www.iana.org/assignments/dns-parameters for assignments and references.
 
 # Do not use these tybesby hashes directly. Use the interface
 # functions, see below.
@@ -88,15 +125,15 @@ use Carp;
     'SIGZERO'   => 0,       # RFC2931 consider this a pseudo type
     'A'         => 1,       # RFC 1035, Section 3.4.1
     'NS'        => 2,       # RFC 1035, Section 3.3.11
-    'MD'        => 3,       # RFC 1035, Section 3.3.4 (obsolete)
-    'MF'        => 4,       # RFC 1035, Section 3.3.5 (obsolete)
+    'MD'        => 3,       # RFC 1035, Section 3.3.4 (obsolete)		NOT IMPLEMENTED
+    'MF'        => 4,       # RFC 1035, Section 3.3.5 (obsolete)		NOT IMPLEMENTED
     'CNAME'     => 5,       # RFC 1035, Section 3.3.1
     'SOA'       => 6,       # RFC 1035, Section 3.3.13
     'MB'        => 7,       # RFC 1035, Section 3.3.3
     'MG'        => 8,       # RFC 1035, Section 3.3.6
     'MR'        => 9,       # RFC 1035, Section 3.3.8
     'NULL'      => 10,      # RFC 1035, Section 3.3.10
-    'WKS'       => 11,      # RFC 1035, Section 3.4.2 (deprecated)
+    'WKS'       => 11,      # RFC 1035, Section 3.4.2 (deprecated)		NOT IMPLEMENTED
     'PTR'       => 12,      # RFC 1035, Section 3.3.12
     'HINFO'     => 13,      # RFC 1035, Section 3.3.2
     'MINFO'     => 14,      # RFC 1035, Section 3.3.7
@@ -108,36 +145,46 @@ use Carp;
     'ISDN'      => 20,      # RFC 1183, Section 3.2
     'RT'        => 21,      # RFC 1183, Section 3.3
     'NSAP'      => 22,      # RFC 1706, Section 5
-    'NSAP_PTR'  => 23,      # RFC 1348 (obsolete)
-    # The following 2 RRs are impemented in Net::DNS::SEC
-    'SIG'       => 24,      # RFC 2535, Section 4.1
-    'KEY'       => 25,      # RFC 2535, Section 3.1
+    'NSAP_PTR'  => 23,      # RFC 1348 (obsolete by RFC 1637)			NOT IMPLEMENTED
+    'SIG'       => 24,      # RFC 2535, Section 4.1				impemented in Net::DNS::SEC
+    'KEY'       => 25,      # RFC 2535, Section 3.1				impemented in Net::DNS::SEC
     'PX'        => 26,      # RFC 2163,
-    'GPOS'      => 27,      # RFC 1712 (obsolete)
+    'GPOS'      => 27,      # RFC 1712 (obsolete ?)				NOT IMPLEMENTED
     'AAAA'      => 28,      # RFC 1886, Section 2.1
     'LOC'       => 29,      # RFC 1876
-    # The following RR is impemented in Net::DNS::SEC
-    'NXT'       => 30,      # RFC 2535, Section 5.2 obsoleted by RFC3755
+    'NXT'       => 30,      # RFC 2535, Section 5.2 obsoleted by RFC3755	impemented in Net::DNS::SEC
     'EID'       => 31,      # draft-ietf-nimrod-dns-xx.txt
     'NIMLOC'    => 32,      # draft-ietf-nimrod-dns-xx.txt
     'SRV'       => 33,      # RFC 2052
-    'ATMA'      => 34,      # ???
+    'ATMA'      => 34,      # non-standard    					NOT IMPLEMENTED
     'NAPTR'     => 35,      # RFC 2168
     'KX'        => 36,      # RFC 2230
     'CERT'      => 37,      # RFC 2538
+    'A6'        => 38,      # RFC3226, RFC2874. See RFC 3363 made A6 exp.	NOT IMPLEMENTED
     'DNAME'     => 39,      # RFC 2672
+    'SINK'      => 40,      # non-standard					NOT IMPLEMENTED
     'OPT'       => 41,      # RFC 2671
-    'DS'        => 43,      # RFC 4034   # in Net::DNS::SEC
-    'SSHFP'     => 44,      # draft-ietf-secsh-dns (No RFC # yet at time of coding)
-#    'IPSECKEY'  => 45,      # RFC 4025
-    'RRSIG'     => 46,      # RFC 4034 in Net::DNS::SEC
-    'NSEC'      => 47,      # RFC 4034 in Net::DNS::SEC
-    'DNSKEY'    => 48,      # RFC 4034 in Net::DNS::SEC
-    'SPF'       => 99,      # rfc-schlitt-spf-classic-o2 (No RFC # yet at time of coding)
+    'APL'       => 42,      # RFC 3123
+    'DS'        => 43,      # RFC 4034  					implemented in Net::DNS::SEC
+    'SSHFP'     => 44,      # RFC 4255
+    'IPSECKEY'  => 45,      # RFC 4025
+    'RRSIG'     => 46,      # RFC 4034 						implemented in Net::DNS::SEC
+    'NSEC'      => 47,      # RFC 4034						implemented in Net::DNS::SEC
+    'DNSKEY'    => 48,      # RFC 4034						inplemented in Net::DNS::SEC
+    'DHCID'     => 49,      # RFC4701
+    'NSEC3'     => 50,      # RFC5155
+    'NSEC3PARAM' => 51,     # RFC5155
+# 52-54 are unassigned
+    'HIP'       => 55,      # RFC5205
+    'NINFO'     => 56,      # non-standard					NOT IMPLEMENTED
+    'RKEY'      => 57,      # non-standard					NOT IMPLEMENTED
+# 58-98 are unasigned
+    'SPF'       => 99,      # RFC 4408
     'UINFO'     => 100,     # non-standard
     'UID'       => 101,     # non-standard
     'GID'       => 102,     # non-standard
     'UNSPEC'    => 103,     # non-standard
+# 104-248 are unasigned
     'TKEY'      => 249,     # RFC 2930
     'TSIG'      => 250,     # RFC 2931
     'IXFR'      => 251,     # RFC 1995
@@ -145,8 +192,8 @@ use Carp;
     'MAILB'     => 253,     # RFC 1035 (MB, MG, MR)
     'MAILA'     => 254,     # RFC 1035 (obsolete - see MX)
     'ANY'       => 255,     # RFC 1035
-    'NSEC3'     => 65324,    # draft-ietf-dnsext-nsec3-07 (experimental typecode, not permanent)
-    'DLV'       => 32769    # RFC 4431  in Net::DNS::SEC		
+    'TA'        => 32768,    # non-standard					NOT IMPLEMENTED
+    'DLV'       => 32769    # RFC 4431						implemented in Net::DNS::SEC
 );
 %typesbyval = reverse %typesbyname;
 
@@ -170,40 +217,31 @@ sub typesbyname {
 
     return $typesbyname{$name} if defined $typesbyname{$name};
 
-    confess "Net::DNS::typesbyname() argument ($name) is not TYPE###" unless 
-        $name =~ m/^\s*TYPE(\d+)\s*$/o;  
-    
-    my $val = $1;
-    
-    confess 'Net::DNS::typesbyname() argument larger than ' . 0xffff if $val > 0xffff;
-    
-    return $val;
+    confess "unknown type $name" unless $name =~ m/TYPE(\d+)/o;
+
+    my $val = 0 + $1;
+    confess 'argument out of range' if $val > 0xffff;
+
+    return $val ? $val : '00';    ## preserve historical behaviour for TYPE0 ##
 }
 
-
-
 sub typesbyval {
     my $val = shift;
-    confess "Net::DNS::typesbyval() argument is not defined" unless defined $val;
-    confess "Net::DNS::typesbyval() argument ($val) is not numeric" unless 
-	$val =~ s/^\s*0*(\d+)\s*$/$1/o;
 
-    
-    
-    return $typesbyval{$val} if $typesbyval{$val};
-    
-    confess 'Net::DNS::typesbyval() argument larger than '. 0xffff if 
-        $val > 0xffff;
-    
+    return $typesbyval{$val} if defined $typesbyval{$val};
+
+    $val += 0;
+    confess 'argument out of range' if $val > 0xffff;
+
     return "TYPE$val";
 }
 
 
 
 #
-# Do not use these classesby hashes directly. See below. 
+# Do not use these classesby hashes directly. See below.
 #
- 
+
 %classesbyname = (
     'IN'        => 1,       # RFC 1035
     'CH'        => 3,       # RFC 1035
@@ -223,30 +261,25 @@ sub typesbyval {
 
 sub classesbyname {
     my $name = uc shift;
-    return $classesbyname{$name} if $classesbyname{$name};
-    
-    confess "Net::DNS::classesbyval() argument is not CLASS### ($name)" unless 
-        $name =~ m/^\s*CLASS(\d+)\s*$/o;
-    
-    my $val = $1;
-    
-    confess 'Net::DNS::classesbyval() argument larger than '. 0xffff if $val > 0xffff;
-    
+
+    return $classesbyname{$name} if defined $classesbyname{$name};
+
+    confess "unknown class $name" unless $name =~ m/CLASS(\d+)/o;
+
+    my $val = 0 + $1;
+    confess 'argument out of range' if $val > 0xffff;
+
     return $val;
 }
 
-
-
 sub classesbyval {
     my $val = shift;
-    
-    confess "Net::DNS::classesbyname() argument is not numeric ($val)" unless 
-	$val =~ s/^\s*0*([0-9]+)\s*$/$1/o;
-    
-    return $classesbyval{$val} if $classesbyval{$val};
-    
-    confess 'Net::DNS::classesbyname() argument larger than ' . 0xffff if $val > 0xffff;
-    
+
+    return $classesbyval{$val} if defined $classesbyval{$val};
+
+    $val += 0;
+    confess 'argument out of range' if $val > 0xffff;
+
     return "CLASS$val";
 }
 
@@ -331,36 +364,29 @@ sub mx {
     # Then we sort the list by preference
     # Then we return it.
     # We do this into an array to force list context.
-    my @ret = sort { $a->preference <=> $b->preference } 
+    my @ret = sort { $a->preference <=> $b->preference }
               grep { $_->type eq 'MX'} $ans->answer;
 
 
     return @ret;
 }
 
-sub yxrrset {
-    return Net::DNS::RR->new_from_string(shift, 'yxrrset');
-}
 
-sub nxrrset {
-    return Net::DNS::RR->new_from_string(shift, 'nxrrset');
-}
+#
+# Auxiliary functions to support dynamic update.
+#
 
-sub yxdomain {
-    return Net::DNS::RR->new_from_string(shift, 'yxdomain');
-}
+sub yxrrset { return new Net::DNS::RR( shift, 'yxrrset' ); }
 
-sub nxdomain {
-    return Net::DNS::RR->new_from_string(shift, 'nxdomain');
-}
+sub nxrrset { return new Net::DNS::RR( shift, 'nxrrset' ); }
 
-sub rr_add {
-    return Net::DNS::RR->new_from_string(shift, 'rr_add');
-}
+sub yxdomain { return new Net::DNS::RR( shift, 'yxdomain' ); }
 
-sub rr_del {
-    return Net::DNS::RR->new_from_string(shift, 'rr_del');
-}
+sub nxdomain { return new Net::DNS::RR( shift, 'nxdomain' ); }
+
+sub rr_add { return new Net::DNS::RR( shift, 'rr_add' ); }
+
+sub rr_del { return new Net::DNS::RR( shift, 'rr_del' ); }
 
 
 
@@ -399,27 +425,27 @@ sub wire2presentation {
     # I start looking forward and do the magic.
 
     my $i=0;
-    
+
     while ($i < $length ){
 	my $char=unpack("x".$i."C1",$wire);
 	if ( $char < 33 || $char > 126 ){
 	    $presentation.= sprintf ("\\%03u" ,$char);
-	}elsif ( $char == ord( "\"" )) {   
-	    $presentation.= "\\\"";    
-	}elsif ( $char == ord( "\$" )) {   
-	    $presentation.= "\\\$";    
-	}elsif ( $char == ord( "(" )) {   
-	    $presentation.= "\\(";    
-	}elsif ( $char == ord( ")" )) {   
-	    $presentation.= "\\)";    
-	}elsif ( $char == ord( ";" )) {   
-	    $presentation.= "\\;";    
-	}elsif ( $char == ord( "@" )) {   
-	    $presentation.= "\\@";    
-	}elsif ( $char == ord( "\\" )) {   
-	    $presentation.= "\\\\" ; 
+	}elsif ( $char == ord( "\"" )) {
+	    $presentation.= "\\\"";
+	}elsif ( $char == ord( "\$" )) {
+	    $presentation.= "\\\$";
+	}elsif ( $char == ord( "(" )) {
+	    $presentation.= "\\(";
+	}elsif ( $char == ord( ")" )) {
+	    $presentation.= "\\)";
+	}elsif ( $char == ord( ";" )) {
+	    $presentation.= "\\;";
+	}elsif ( $char == ord( "@" )) {
+	    $presentation.= "\\@";
+	}elsif ( $char == ord( "\\" )) {
+	    $presentation.= "\\\\" ;
 	}elsif ( $char==ord (".") ){
-	    $presentation.= "\\." ; 
+	    $presentation.= "\\." ;
 	}else{
 	    $presentation.=chr($char) 	;
 	}
@@ -427,7 +453,24 @@ sub wire2presentation {
     }
 
     return $presentation;
-    
+
+}
+
+
+
+
+sub stripdot {
+	# Code courtesy of JMEHNLE <JMEHNLE@cpan.org>
+	# rt.cpan.org #51009
+
+	# Strips the final non-escaped dot from a domain name.  Note
+	# that one could have a label that looks like "foo\\\\\.\.."
+	# although not likely one wants to deal with that cracefully.
+	# This utilizes 2 functions in the DNS module to deal with
+	# thing cracefully.
+
+	return join('.', map(wire2presentation($_), name2labels(shift)));
+
 }
 
 
@@ -442,52 +485,48 @@ sub wire2presentation {
 sub presentation2wire {
     my  $presentation=shift;
     my  $wire="";
-    my $length=length($presentation);
-    
-    my $i=0;
-    
-    while ($i < $length ){
-	my $char=unpack("x".$i."C1",$presentation);
-	if (  $char == ord ('.')){
-	    return ($wire,substr($presentation,$i+1));
-	}
-	if (  $char == ord ('\\')){
-	    #backslash found
-	    pos($presentation)=$i+1;
-	    if ($presentation=~/\G(\d\d\d)/){
-		$wire.=pack("C",$1);
-		$i+=3;
-	    }elsif($presentation=~/\Gx([0..9a..fA..F][0..9a..fA..F])/){
-		$wire.=pack("H*",$1);
-		$i+=3;
-	    }elsif($presentation=~/\G\./){
-		$wire.="\.";
-		$i+=1;
-	    }elsif($presentation=~/\G@/){
-		$wire.="@";
-		$i+=1;
-	    }elsif($presentation=~/\G\(/){
-		$wire.="(";
-		$i+=1;
-	    }elsif($presentation=~/\G\)/){
-		$wire.=")";
-		$i+=1;
-           }elsif($presentation=~/\G\\/){
-               $wire.="\\"; 
-               $i+=1;
+
+    while ($presentation =~ /\G([^.\\]*)([.\\]?)/g){
+        $wire .= $1 if defined $1;
+
+        if ($2) {
+            if ($2 eq '.') {
+                return ($wire,substr($presentation,pos $presentation));
 	    }
-	}else{
-	    $wire .=  pack("C",$char);  
+
+            #backslash found
+            if ($presentation =~ /\G(\d\d\d)/gc) {
+                $wire.=pack("C",$1);
+            } elsif ($presentation =~ /\G([@().\\])/gc){
+                $wire .= $1;
+            }
         }
-	$i++;
     }
-    
+
     return $wire;
 }
 
 
 
 
+#
+# Auxiliary functions to support policy-driven zone serial numbering.
+#
+#	$successor = $soa->serial(SEQUENTIAL);
+#	$successor = $soa->serial(UNIXTIME);
+#	$successor = $soa->serial(YYYYMMDDxx);
+#
+
+sub SEQUENTIAL { undef }
+
+sub UNIXTIME { return CORE::time; }
+
+sub YYYYMMDDxx {
+	my ( $dd, $mm, $yy ) = ( localtime )[3 .. 5];
+	return 1900010000 + sprintf '%d%0.2d%0.2d00', $yy, $mm, $dd;
+}
+
+
 
 sub rrsort {
     my ($rrtype,$attribute,@rr_array)=@_;
@@ -501,7 +540,7 @@ sub rrsort {
     }
 
     # attribute is empty or not specified.
-    
+
     if( ref($attribute)=~/^Net::DNS::RR::.*/){
 	# push the attribute back on the array.
 	push @rr_array,$attribute;
@@ -516,8 +555,8 @@ sub rrsort {
     return () unless  @extracted_rr;
     my $func=("Net::DNS::RR::".$rrtype)->get_rrsort_func($attribute);
     my @sorted=sort $func  @extracted_rr;
-    return @sorted; 
-    
+    return @sorted;
+
 }
 
 
@@ -531,24 +570,7 @@ sub rrsort {
 1;
 __END__
 
-=head1 NAME
 
-Net::DNS - Perl interface to the DNS resolver
-
-=head1 SYNOPSIS
-
-C<use Net::DNS;>
-
-=head1 DESCRIPTION
-
-Net::DNS is a collection of Perl modules that act as a Domain
-Name System (DNS) resolver.  It allows the programmer to perform
-DNS queries that are beyond the capabilities of C<gethostbyname>
-and C<gethostbyaddr>.
-
-The programmer should be somewhat familiar with the format of
-a DNS packet and its various sections.  See RFC 1035 or
-I<DNS and BIND> (Albitz & Liu) for details.
 
 =head2 Resolver Objects
 
@@ -593,7 +615,7 @@ The additional section, a list of L<Net::DNS::RR|Net::DNS::RR> objects.
 
 The L<Net::DNS::Update|Net::DNS::Update> package is a subclass of
 L<Net::DNS::Packet|Net::DNS::Packet> for creating packet objects to be
-used in dynamic updates.  
+used in dynamic updates.
 
 =head2 Header Objects
 
@@ -743,12 +765,43 @@ Returns a C<Net::DNS::RR> object or C<undef> if the object couldn't
 be created.
 
 
+
+=head1 Zone Serial Number Management
+
+The Net::DNS module provides auxiliary functions which support
+policy-driven zone serial numbering regimes.
+
+=head2 Strictly Sequential
+
+    $successor = $soa->serial( SEQUENTIAL );
+
+The existing serial number is incremented modulo 2**32.
+
+=head2 Time Encoded
+
+    $successor = $soa->serial( UNIXTIME );
+
+The Unix time scale will be used as the basis for zone serial
+numbering. The serial number will be incremented if the time
+elapsed since the previous update is less than one second.
+
+=head2 Date Encoded
+
+    $successor = $soa->serial( YYYYMMDDxx );
+
+The 32 bit value returned by the auxiliary YYYYMMDDxx() function
+will be used as the base for the date-coded zone serial number.
+Serial number increments must be limited to 100 per day for the
+date information to remain useful.
+
+
+
 =head2 Sorting of RR arrays
 
 As of version 0.55 there is functionality to help you sort RR
 arrays. 'rrsort()' is the function that is available to do the
 sorting. In most cases rrsort will give you the answer that you
-want but you can specify your own sorting method by using the 
+want but you can specify your own sorting method by using the
 Net::DNS::RR::FOO->set_rrsort_func() class method. See L<Net::DNS::RR>
 for details.
 
@@ -766,7 +819,7 @@ the attribute that is specified as second argument.
 There are a number of RRs for which the sorting function is
 specifically defined for certain attributes.  If such sorting function
 is defined in the code (it can be set or overwritten using the
-set_rrsort_func() class method) that function is used. 
+set_rrsort_func() class method) that function is used.
 
 For instance:
    my @prioritysorted=rrsort("SRV","priority",@rr_array);
@@ -774,7 +827,7 @@ returns the SRV records sorted from lowest to heighest priority and
 for equal priorities from heighes to lowes weight.
 
 If the function does not exist then a numerical sort on the attribute
-value is performed. 
+value is performed.
    my @portsorted=rrsort("SRV","port",@rr_array);
 
 If the attribute does not exist for a certain RR than the RRs are
@@ -805,7 +858,7 @@ dynamic updates.
   use Net::DNS;
   my $res   = Net::DNS::Resolver->new;
   my $query = $res->search("host.example.com");
-  
+
   if ($query) {
       foreach my $rr ($query->answer) {
           next unless $rr->type eq "A";
@@ -820,7 +873,7 @@ dynamic updates.
   use Net::DNS;
   my $res   = Net::DNS::Resolver->new;
   my $query = $res->query("example.com", "NS");
-  
+
   if ($query) {
       foreach $rr (grep { $_->type eq 'NS' } $query->answer) {
           print $rr->nsdname, "\n";
@@ -836,7 +889,7 @@ dynamic updates.
   my $name = "example.com";
   my $res  = Net::DNS::Resolver->new;
   my @mx   = mx($res, $name);
-  
+
   if (@mx) {
       foreach $rr (@mx) {
           print $rr->preference, " ", $rr->exchange, "\n";
@@ -851,7 +904,7 @@ dynamic updates.
   use Net::DNS;
   my $res   = Net::DNS::Resolver->new;
   my $query = $res->query("example.com", "SOA");
-  
+
   if ($query) {
       ($query->answer)[0]->print;
   } else {
@@ -863,9 +916,9 @@ dynamic updates.
   use Net::DNS;
   my $res  = Net::DNS::Resolver->new;
   $res->nameservers("ns.example.com");
-  
+
   my @zone = $res->axfr("example.com");
-  
+
   foreach $rr (@zone) {
       $rr->print;
   }
@@ -891,12 +944,12 @@ has arrived.
 
   use Net::DNS;
   use IO::Select;
-  
+
   my $timeout = 5;
   my $res     = Net::DNS::Resolver->new;
   my $bgsock  = $res->bgsend("host.example.com");
   my $sel     = IO::Select->new($bgsock);
-  
+
   # Add more sockets to $sel if desired.
   my @ready = $sel->can_read($timeout);
   if (@ready) {
@@ -914,22 +967,27 @@ has arrived.
       warn "timed out after $timeout seconds\n";
   }
 
+
 =head1 BUGS
 
 C<Net::DNS> is slow.
 
-For other items to be fixed, please see the TODO file included with
-the source distribution.
+For other items to be fixed, or if you discover a bug in this
+distribution please use the CPAN bug reporting system.
+
 
 =head1 COPYRIGHT
 
-Copyright (c) 1997-2002 Michael Fuhr. 
-Portions Copyright (c) 2002-2004 Chris Reinhardt.
-Portions Copyright (c) 2005 Olaf Kolkman (RIPE NCC)
-Portions Copyright (c) 2006 Olaf Kolkman (NLnet Labs)
+Copyright (c)1997-2002 Michael Fuhr.
+Portions Copyright(c)2002-2004 Chris Reinhardt.
+Portions Copyright(c)2005 Olaf Kolkman (RIPE NCC)
+Portions Copyright(c)2006 Olaf Kolkman (NLnet Labs)
+
+All rights reserved.
+
+This program is free software; you may redistribute it and/or
+modify it under the same terms as Perl itself.
 
-All rights reserved.  This program is free software; you may redistribute
-it and/or modify it under the same terms as Perl itself.
 
 =head1 AUTHOR INFORMATION
 
@@ -943,20 +1001,21 @@ Between 2002 and 2004 Net::DNS was maintained by:
 
 Net::DNS was created by:
 	Michael Fuhr
-	mike@fuhr.org 
+	mike@fuhr.org
 
 
 
 For more information see:
     http://www.net-dns.org/
 
-Stay tuned and syncicate:
+Stay tuned and syndicate:
     http://www.net-dns.org/blog/
 
 =head1 SEE ALSO
- 
-L<perl(1)>, L<Net::DNS::Resolver>, L<Net::DNS::Packet>, L<Net::DNS::Update>,
+
+L<perl>, L<Net::DNS::Resolver>, L<Net::DNS::Packet>, L<Net::DNS::Update>,
 L<Net::DNS::Header>, L<Net::DNS::Question>, L<Net::DNS::RR>, RFC 1035,
 I<DNS and BIND> by Paul Albitz & Cricket Liu
 
 =cut
+
