@@ -1,46 +1,46 @@
 package Net::DNS::RR;
+
 #
-# $Id: RR.pm 828 2009-12-23 15:03:29Z olaf $
+# $Id: RR.pm 969 2011-12-13 10:34:39Z willem $
 #
-use strict;
+use vars qw($VERSION);
+$VERSION = (qw$LastChangedRevision: 969 $)[1];
 
-BEGIN { 
-    eval { require bytes; }
-} 
-
-
-use vars qw($VERSION $AUTOLOAD %rrsortfunct );
-use Carp;
-use Net::DNS qw (wire2presentation name2labels stripdot);
-use Net::DNS::RR::Unknown;
-
-
-$VERSION = (qw$LastChangedRevision: 828 $)[1];
 
 =head1 NAME
 
-Net::DNS::RR - DNS Resource Record class
+Net::DNS::RR - DNS Resource Record base class
 
 =head1 SYNOPSIS
 
-C<use Net::DNS::RR>
+use Net::DNS;
+
+    $rr = new Net::DNS::RR('example.com IN A 192.0.2.99');
+
+    $rr = new Net::DNS::RR(
+	    name    => 'example.com'
+	    type    => 'A',
+	    address => '192.0.2.99'
+	    );
+
 
 =head1 DESCRIPTION
 
 C<Net::DNS::RR> is the base class for DNS Resource Record (RR) objects.
 See also the manual pages for each RR type.
 
-=head1 METHODS
-
-B<WARNING!!!>  Don't assume the RR objects you receive from a query
-are of a particular type -- always check an object's type before calling
-any of its methods.  If you call an unknown method, you'll get a nasty
-warning message and C<Net::DNS::RR> will return C<undef> to the caller.
-
 =cut
 
+
+use strict;
+use integer;
+use Carp;
+use Net::DNS qw (wire2presentation name2labels stripdot);
+use Net::DNS::RR::Unknown;
+
+
 # %RR needs to be available within the scope of the BEGIN block.
-# $RR_REGEX is a global just to be on the safe side.  
+# $RR_REGEX is a global just to be on the safe side.
 # %_LOADED is used internally for autoloading the RR subclasses.
 use vars qw(%RR %_LOADED $RR_REGEX);
 
@@ -90,38 +90,38 @@ BEGIN {
 
 	#  Only load DNSSEC if available
 
-	eval { 	    
+	eval {
 	    local $SIG{'__DIE__'} = 'DEFAULT';
-	    require Net::DNS::RR::SIG; 
+	    require Net::DNS::RR::SIG;
 	};
 
 	unless ($@) {
 		$RR{'SIG'} = 1;
-		eval { 	    
+		eval {
 		    local $SIG{'__DIE__'} = 'DEFAULT';
-		    require Net::DNS::RR::NXT; 
+		    require Net::DNS::RR::NXT;
 		};
-		
+
 		unless ($@) {
 		    $RR{'NXT'}	= 1;
 		} else {
 		    die $@;
 		}
-		
-		eval { 
+
+		eval {
 		    local $SIG{'__DIE__'} = 'DEFAULT';
-		    require Net::DNS::RR::KEY; 
+		    require Net::DNS::RR::KEY;
 		};
-		
+
 		unless ($@) {
 		    $RR{'KEY'} = 1;
 		} else {
 		    die $@;
 		}
 
-	 	eval { 
+	 	eval {
 		    local $SIG{'__DIE__'} = 'DEFAULT';
-		    require Net::DNS::RR::DS; 
+		    require Net::DNS::RR::DS;
 		};
 
 	 	unless ($@) {
@@ -131,26 +131,26 @@ BEGIN {
 		    die $@;
 		}
 
-	 	eval { 
+	 	eval {
 		    local $SIG{'__DIE__'} = 'DEFAULT';
-		    require Net::DNS::RR::RRSIG; 
+		    require Net::DNS::RR::RRSIG;
 		};
 
 	 	unless ($@) {
 		    $RR{'RRSIG'} = 1;
 		    # If RRSIG is available so should the other DNSSEC types
-		    eval {		    
+		    eval {
 			local $SIG{'__DIE__'} = 'DEFAULT';
-			require Net::DNS::RR::NSEC; 
+			require Net::DNS::RR::NSEC;
 		    };
 		    unless ($@) {
 		      $RR{'NSEC'} = 1;
 		    } else {
 		    die $@;
 		  }
-		    eval { 
+		    eval {
 			local $SIG{'__DIE__'} = 'DEFAULT';
-			require Net::DNS::RR::DNSKEY; 
+			require Net::DNS::RR::DNSKEY;
 		    };
 
 		    unless ($@) {
@@ -158,45 +158,45 @@ BEGIN {
 		    } else {
 		      die $@;
 		    }
-		} 
+		}
 
-	 	eval { 
+	 	eval {
 		  local $SIG{'__DIE__'} = 'DEFAULT';
-		  require Net::DNS::RR::DLV; 
+		  require Net::DNS::RR::DLV;
 		};
 
 		unless ($@) {
 		  $RR{'DLV'} =1;
 		} else {
-		  # Die only if we are dealing with a version for which DLV is 
-		  # available 
+		  # Die only if we are dealing with a version for which DLV is
+		  # available
 		  die $@ if defined ($Net::DNS::SEC::HAS_DLV) ;
 
 		}
 
-	 	eval { 
+	 	eval {
 		  local $SIG{'__DIE__'} = 'DEFAULT';
-		  require Net::DNS::RR::NSEC3; 
+		  require Net::DNS::RR::NSEC3;
 		};
 
 		unless ($@) {
 		  $RR{'NSEC3'} =1;
 		} else {
-		  # Die only if we are dealing with a version for which NSEC3 is		  # available 
+		  # Die only if we are dealing with a version for which NSEC3 is		  # available
 		  die $@ if defined ($Net::DNS::SEC::HAS_NSEC3);
 		}
-		
-		
-	 	eval { 
+
+
+	 	eval {
 		  local $SIG{'__DIE__'} = 'DEFAULT';
-		  require Net::DNS::RR::NSEC3PARAM; 
+		  require Net::DNS::RR::NSEC3PARAM;
 		};
 
 		unless ($@) {
 		  $RR{'NSEC3PARAM'} =1;
 		} else {
-		  # Die only if we are dealing with a version for which NSEC3 is 
-		  # available 
+		  # Die only if we are dealing with a version for which NSEC3 is
+		  # available
 
 		  die $@ if defined($Net::DNS::SEC::SVNVERSION) &&  $Net::DNS::SEC::SVNVERSION > 619;   # In the code since. (for users of the SVN trunk)
 		}
@@ -213,12 +213,12 @@ sub build_regex {
 	my $types   = join('|', sort { length $b <=> length $a } keys %Net::DNS::typesbyname);
 
 	$types .= '|TYPE\\d+';
-				
-	$RR_REGEX   = " ^ 
+
+	$RR_REGEX   = " ^
 					\\s*
-    	            (\\S+) # name anything non-space will do 
-    	            \\s*                
-    	            (\\d+)?           
+    	            (\\S+) # name anything non-space will do
+    	            \\s*
+    	            (\\d+)?
     	            \\s*
     	            ($classes)?
     	            \\s*
@@ -231,6 +231,14 @@ sub build_regex {
 }
 
 
+=head1 METHODS
+
+B<WARNING!!!>  Do not assume the RR objects you receive from a query
+are of a particular type -- always check the object type before calling
+any of its methods.  If you call an unknown method, you will get an
+error message and execution will be terminated.
+
+
 =head2 new (from string)
 
  $a     = Net::DNS::RR->new("foo.example.com. 86400 A 10.1.2.3");
@@ -260,7 +268,7 @@ All names must be fully qualified.  The trailing dot (.) is optional.
 	 type    => "A",
 	 address => "10.1.2.3",
  );
- 
+
  $rr = Net::DNS::RR->new(
 	 name => "foo.example.com",
 	 type => "A",
@@ -329,7 +337,7 @@ sub new_from_string {
 		if ($rrstring=~s/^(\\.)//o){             # Escaped special character
 			$cleanstring.=$1;
 		}
-		
+
 		if ($rrstring=~s/^((['"]).*(?<!\\)\2)//o){ # Anything within a matching string block. (non escaped terminator)
 			$cleanstring.=$1;
 
@@ -340,22 +348,22 @@ sub new_from_string {
 		}elsif ($rrstring=~s/^((['"]).*((?<!\\);)?)//o){
 			$cleanstring.=$1;
 		}
-		
+
 		$rrstring=~s/^(;.*\n)//o;  # comment till newline
 		$rrstring=~s/^(;.*$)//o;   # comment till end of string
 		#print STDERR ".";
 
-		confess "Failed stripping:loop will not terminate. Please report this info: ". $cleanstring ."---". $rrstring."\n" 
+		confess "Failed stripping:loop will not terminate. Please report this info: ". $cleanstring ."---". $rrstring."\n"
 		  if ($loopdetection==length($rrstring));
 		$loopdetection=length($rrstring);
 
 
 	}
-	  
-	
-	
 
-	($cleanstring =~ m/$RR_REGEX/xso) || 
+
+
+
+	($cleanstring =~ m/$RR_REGEX/xso) ||
 		confess qq|Internal Error: "$rrstring" did not match RR pat.\nPlease report this to the author!\n|;
 
 	my $name    = $1;
@@ -386,11 +394,11 @@ sub new_from_string {
 	}
 
 	$rrtype ||= 'ANY';
-	
+
 
 	if ($update_type) {
 		$update_type = lc $update_type;
-		
+
 		if ($update_type eq 'yxrrset') {
 			$ttl     = 0;
 			$rrclass = 'ANY' unless $rdata;
@@ -435,33 +443,10 @@ sub new_from_string {
 		$rdata =~ m/\\\#\s+(\d+)\s+(.*)$/o;
 
 		my $rdlength = $1;
-		my $hexdump  = $2;		
+		my $hexdump  = $2;
 		$hexdump =~ s/\s*//og;
 
-		die "$rdata is inconsistent; length does not match content" 
-			if length($hexdump) != $rdlength*2;
-
-		$rdata = pack('H*', $hexdump);
-	
-		return Net::DNS::RR->new_from_data(
-			$name, 
-			$rrtype, 
-			$rrclass, 
-			$ttl, 
-			$rdlength, 
-			\$rdata, 
-			length($rdata) - $rdlength
-		);
-	} elsif ($rdata=~/\s*\\\#\s+\d+\s+/o) {   
-		#We are now dealing with the truly unknown.
-		die 'Expected RFC3597 representation of RDATA' 
-			unless $rdata =~ m/\\\#\s+(\d+)\s+(.*)$/o;
-
-		my $rdlength = $1;
-		my $hexdump  = $2;		
-		$hexdump =~ s/\s*//og;
-
-		die "$rdata is inconsistent; length does not match content" 
+		die "$rdata is inconsistent; length does not match content"
 			if length($hexdump) != $rdlength*2;
 
 		$rdata = pack('H*', $hexdump);
@@ -475,12 +460,35 @@ sub new_from_string {
 			\$rdata,
 			length($rdata) - $rdlength
 		);
-	} else {  
+	} elsif ($rdata=~/\s*\\\#\s+\d+\s+/o) {
+		#We are now dealing with the truly unknown.
+		die 'Expected RFC3597 representation of RDATA'
+			unless $rdata =~ m/\\\#\s+(\d+)\s+(.*)$/o;
+
+		my $rdlength = $1;
+		my $hexdump  = $2;
+		$hexdump =~ s/\s*//og;
+
+		die "$rdata is inconsistent; length does not match content"
+			if length($hexdump) != $rdlength*2;
+
+		$rdata = pack('H*', $hexdump);
+
+		return Net::DNS::RR->new_from_data(
+			$name,
+			$rrtype,
+			$rrclass,
+			$ttl,
+			$rdlength,
+			\$rdata,
+			length($rdata) - $rdlength
+		);
+	} else {
 		#God knows how to handle these... bless them in the RR class.
 		bless $self, $class;
 		return $self
 	}
-	
+
 }
 
 sub new_from_hash {
@@ -510,9 +518,9 @@ sub new_from_hash {
 			$self->_normalize_dnames();
 			return _normalize_rdata($self);
 
-	    } else {  
+	    } else {
 			# Special processing of OPT. Since TTL and CLASS are
-			# set by other variables. See Net::DNS::RR::OPT 
+			# set by other variables. See Net::DNS::RR::OPT
 			# documentation
 			return $subclass->new_from_hash($self);
 	    }
@@ -548,7 +556,7 @@ sub _normalize_rdata {
 			answer		=> [],
 			authority	=> [],
 			additional	=> []	};
-	
+
 	bless $pkt, "Net::DNS::Packet";
 	$pkt->push( answer => $self );
 	my $pkt2 = Net::DNS::Packet->new( \$pkt->data );
@@ -569,34 +577,34 @@ sub _normalize_dnames {
 
 
 sub _normalize_ownername {
-	my $self=shift;	
+	my $self=shift;
 	return $self->{'name'}=stripdot($self->{'name'});
 }
 
 
-=head2 parse
+=head2 decode
 
-    ($rrobj, $offset) = Net::DNS::RR->parse(\$data, $offset);
+    ($rrobj, $offset) = Net::DNS::RR->decode(\$data, $offset);
 
-Parses a DNS resource record at the specified location within a DNS packet.
+Decodes a DNS resource record at the specified location within a DNS packet.
 The first argument is a reference to the packet data.
 The second argument is the offset within the packet where the resource record begins.
 
 Returns a Net::DNS::RR object and the offset of the next location in the packet.
 
-Parsing is aborted if the object could not be created (e.g., corrupt or insufficient data).
+Decoding is aborted if the object could not be created (e.g., corrupt or insufficient data).
 
 =cut
 
-use constant PACKED_LENGTH => length pack 'n2 N n', (0)x4;
+use constant RRFIXEDSZ => length pack 'n2 N n', (0) x 4;
 
-sub parse {
+sub decode {
 	my ($objclass, $data, $offset) = @_;
 
-	my ($name, $index) = Net::DNS::Packet::dn_expand($data, $offset);
+	my ($name, $index) = Net::DNS::Packet::dn_expand($data, $offset || 0);
 	die 'Exception: corrupt or incomplete data' unless $index;
 
-	my $rdindex = $index + PACKED_LENGTH;
+	my $rdindex = $index + RRFIXEDSZ;
 	die 'Exception: incomplete data' if length $$data < $rdindex;
 	my ($type, $class, $ttl, $rdlength) = unpack("\@$index n2 N n", $$data);
 
@@ -619,12 +627,6 @@ sub parse {
 }
 
 
-#
-# Some people have reported that Net::DNS dies because AUTOLOAD picks up
-# calls to DESTROY.
-#
-sub DESTROY {}
-
 =head2 print
 
     $rr->print;
@@ -654,7 +656,7 @@ B<name>, which will not return the trailing dot.
 sub string {
 	my $self = shift;
 	my $data = $self->rdatastr || '; no data';
-  
+
 	join "\t", "$self->{name}.", $self->{ttl}, $self->{class}, $self->{type}, $data;
 }
 
@@ -709,7 +711,7 @@ sub class {
 	}
 	return $self->{'class'};
 }
-	
+
 
 =head2 ttl
 
@@ -751,19 +753,22 @@ sub rr_rdata {
 	return exists $self->{'rdata'} ? $self->{'rdata'} : '';
 }
 
+
 #------------------------------------------------------------------------------
-# sub data
+# sub encode
 #
 # This method is called by Net::DNS::Packet->data to get the binary
 # representation of an RR.
 #------------------------------------------------------------------------------
 
-sub data {
-	my ($self, $packet, $offset) = @_;
-	my $data;
-
+sub encode {
+	my ( $self, $offset, $hash, $packet ) = @_;
+	$offset ||= 0;
+	$packet ||= bless {}, qw(Net::DNS::Packet);
+	$packet->{compnames} = $hash || {};
 
 	# Don't compress TSIG or TKEY names and don't mess with EDNS0 packets
+	my $data;
 	if (uc($self->{'type'}) eq 'TSIG' || uc($self->{'type'}) eq 'TKEY') {
 		my $tmp_packet = Net::DNS::Packet->new();
 		$data = $tmp_packet->dn_comp($self->{'name'}, 0);
@@ -773,7 +778,7 @@ sub data {
 		$data = $tmp_packet->dn_comp('', 0);
 	} else {
 		$data  = $packet->dn_comp($self->{'name'}, $offset);
-		return undef unless defined $data;	
+		return undef unless defined $data;
 	}
 
 	my $qtype     = uc($self->{'type'});
@@ -784,7 +789,7 @@ sub data {
 	my $qclass_val = ($qclass =~ m/^\d+$/o) ? $qclass : Net::DNS::classesbyname($qclass);
 	$qclass_val    = 0 if !defined($qclass_val);
 	$data .= pack('n', $qtype_val);
-	
+
 	# If the type is OPT then class will need to contain a decimal number
 	# containing the UDP payload size. (RFC2671 section 4.3)
 	if (uc($self->{'type'}) ne 'OPT') {
@@ -792,7 +797,7 @@ sub data {
 	} else {
 	    $data .= pack('n', $self->{'class'});
 	}
-	
+
 	$data .= pack('N', $self->{'ttl'});
 
 	$offset += length($data) + &Net::DNS::INT16SZ;	# allow for rdlength
@@ -810,16 +815,18 @@ sub data {
 
 
 #------------------------------------------------------------------------------
-#  This method is called by SIG objects verify method. 
+#  This method is called by SIG objects verify method.
 #  It is almost the same as data but needed to get an representation of the
 #  packets in wire format withoud domain name compression.
 #  It is essential to DNSSEC RFC 2535 section 8
 #------------------------------------------------------------------------------
 
+sub canonical {&_canonicaldata}
+
 sub _canonicaldata {
 	my $self = shift;
 	my $data='';
-	{   
+	{
 	    my $name=$self->{'name'};
 	    my @dname=Net::DNS::name2labels($name);
 	    for (my $i=0;$i<@dname;$i++){
@@ -831,10 +838,10 @@ sub _canonicaldata {
 	$data .= pack('n', Net::DNS::typesbyname(uc($self->{'type'})));
 	$data .= pack('n', Net::DNS::classesbyname(uc($self->{'class'})));
 	$data .= pack('N', $self->{'ttl'});
-	
-	
+
+
 	my $rdata = $self->_canonicalRdata;
-	
+
 	$data .= pack('n', length $rdata);
 	$data .= $rdata;
 	return $data;
@@ -861,7 +868,7 @@ sub _canonicalRdata {
 
 
 
-sub _name2wire   {   
+sub _name2wire   {
     my ($self, $name) = @_;
 
     my $rdata="";
@@ -873,7 +880,7 @@ sub _name2wire   {
 		$rdata .= pack('C', length $_);
 		$rdata .= $_ ;
     }
-    
+
     $rdata .= pack('C', '0');
     return $rdata;
 }
@@ -881,6 +888,7 @@ sub _name2wire   {
 
 
 
+use vars qw($AUTOLOAD);
 
 sub AUTOLOAD {
 	my ($self) = @_;  # If we do shift here, it will mess up the goto below.
@@ -891,11 +899,11 @@ sub AUTOLOAD {
 	if ($name =~ /get_rrsort_func/){
 	    return Net::DNS::RR::get_rrsort_func(@_);
 	}
-	# XXX -- We should test that we do in fact carp on unknown methods.	
+	# XXX -- We should test that we do in fact carp on unknown methods.
 	unless (exists $self->{$name}) {
 	    my $rr_string = $self->string;
 	    Carp::carp(<<"AMEN");
-	    
+
 ***
 ***  WARNING!!!  The program has attempted to call the method
 ***  "$name" for the following RR object:
@@ -909,29 +917,30 @@ sub AUTOLOAD {
 ***  calling any of its methods.
 ***
 ***  Net::DNS has returned undef to the caller.
-*** 
+***
 
 AMEN
 return;
 	}
-	
+
 	no strict q/refs/;
-	
+
 	# Build a method in the class.
 	*{$AUTOLOAD} = sub {
 	    my ($self, $new_val) = @_;
-	    
+
 	    if (defined $new_val) {
 		$self->{$name} = $new_val;
 	    }
-	    
+
 	    return $self->{$name};
 	};
-	
+
 	# And jump over to it.
 	goto &{$AUTOLOAD};
 }
 
+sub DESTROY {}
 
 
 #
@@ -941,19 +950,19 @@ return;
 #
 sub _get_subclass {
 	my ($class, $type) = @_;
-	
+
 	return unless $type and $RR{$type};
-	
+
 	my $subclass = join('::', $class, $type);
-	
+
 	unless ($_LOADED{$subclass}) {
 		eval "require $subclass";
 		die $@ if $@;
 		$_LOADED{$subclass}++;
 	}
-	
+
 	return $subclass;
-}	
+}
 
 
 
@@ -998,13 +1007,14 @@ objects from the class you called the set_prop_sort from. In other
 words, you can rest assured that the above sorting function will only
 get Net::DNS::RR::SRV objects.
 
-The above example is the sorting function that actually is implemented in 
+The above example is the sorting function that actually is implemented in
 SRV.
 
 =cut
 
 
 
+use vars qw(%rrsortfunct);
 
 sub set_rrsort_func{
     my $class=shift;
@@ -1014,9 +1024,9 @@ sub set_rrsort_func{
     my ($type) = $class =~ m/^.*::(.*)$/o;
     $Net::DNS::RR::rrsortfunct{$type}{$attribute}=$funct;
 }
-		
+
 sub get_rrsort_func {
-    my $class=shift;    
+    my $class=shift;
     my $attribute=shift;  #can be undefined.
     my $sortsub;
     my ($type) = $class =~ m/^.*::(.*)$/o;
@@ -1041,10 +1051,10 @@ sub get_rrsort_func {
 	return $Net::DNS::RR::rrsortfunct{$type}{'default_sort'};
     }
     elsif( defined($attribute) ){
-	
+
 	return sub{
 	    my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
-	    ( exists($a->{$attribute}) &&   
+	    ( exists($a->{$attribute}) &&
 	      $a->{$attribute} <=> $b->{$attribute})
 		||
 		$a->_canonicaldata() cmp $b->_canonicaldata()
@@ -1054,7 +1064,7 @@ sub get_rrsort_func {
 	    my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
 	    $a->_canonicaldata() cmp $b->_canonicaldata()
 	};
-    }    
+    }
 
     return $sortsub;
 }
@@ -1063,12 +1073,12 @@ sub get_rrsort_func {
 
 
 
-	
+
 sub STORABLE_freeze {
 	my ($self, $cloning) = @_;
 
 	return if $cloning;
-	
+
 	return ('', {%$self});
 }
 
@@ -1076,12 +1086,27 @@ sub STORABLE_thaw {
 	my ($self, $cloning, undef, $data) = @_;
 
 	%{$self}  = %{$data};
-	
+
 	__PACKAGE__->_get_subclass($self->{'type'});
-	
+
 	return $self;
 }
 
+
+#
+#	dump
+#
+#	    $rr->dump;
+#
+#	Prints a depth-first recursive listing of the record data structure.
+#
+
+sub dump {				## print internal data structure
+	use Data::Dumper;
+	print Dumper(shift);
+}
+
+
 =head1 BUGS
 
 This version of C<Net::DNS::RR> does little sanity checking on user-created
@@ -1089,22 +1114,24 @@ RR objects.
 
 =head1 COPYRIGHT
 
-Copyright (c) 1997-2002 Michael Fuhr. 
+Copyright (c)1997-2002 Michael Fuhr.
 
-Portions Copyright (c) 2002-2004 Chris Reinhardt.
+Portions Copyright (c)2002-2004 Chris Reinhardt.
 
-Portions Copyright (c) 2005-2007 Olaf Kolkman 
+Portions Copyright (c)2005-2007 Olaf Kolkman
 
-Portions Copyright (c) 2007 Dick Franks 
+Portions Copyright (c)2007 Dick Franks
 
-All rights reserved.  This program is free software; you may redistribute
-it and/or modify it under the same terms as Perl itself.
+All rights reserved.
+
+This program is free software; you may redistribute it and/or
+modify it under the same terms as Perl itself.
 
 EDNS0 extensions by Olaf Kolkman.
 
 =head1 SEE ALSO
 
-L<perl(1)>, L<Net::DNS>, L<Net::DNS::Resolver>, L<Net::DNS::Packet>,
+L<perl>, L<Net::DNS>, L<Net::DNS::Resolver>, L<Net::DNS::Packet>,
 L<Net::DNS::Update>, L<Net::DNS::Header>, L<Net::DNS::Question>,
 RFC 1035 Section 4.1.3
 
