@@ -1,12 +1,15 @@
 package Net::DNS::RR::EID;
 #
-# $Id: EID.pm 102 2004-08-12 05:16:06Z ctriv $
+# $Id: EID.pm 932 2011-10-26 12:40:48Z willem $
 #
 use strict;
+BEGIN {
+    eval { require bytes; }
+}
 use vars qw(@ISA $VERSION);
 
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$LastChangedRevision: 102 $)[1];
+$VERSION = (qw$LastChangedRevision: 932 $)[1];
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
@@ -44,7 +47,7 @@ Returns the record's data section as binary data.
 
 =head1 COPYRIGHT
 
-Copyright (c) 1997-2002 Michael Fuhr. 
+Copyright (c) 1997-2002 Michael Fuhr.
 
 Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
