@@ -1,12 +1,15 @@
 package Net::DNS::RR::MG;
 #
-# $Id: MG.pm 102 2004-08-12 05:16:06Z ctriv $
+# $Id: MG.pm 932 2011-10-26 12:40:48Z willem $
 #
 use strict;
+BEGIN {
+    eval { require bytes; }
+}
 use vars qw(@ISA $VERSION);
 
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$LastChangedRevision: 102 $)[1];
+$VERSION = (qw$LastChangedRevision: 932 $)[1];
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
@@ -14,7 +17,7 @@ sub new {
 	if ($self->{"rdlength"} > 0) {
 		($self->{"mgmname"}) = Net::DNS::Packet::dn_expand($data, $offset);
 	}
-	
+
 	return bless $self, $class;
 }
 
@@ -22,8 +25,7 @@ sub new_from_string {
 	my ($class, $self, $string) = @_;
 
 	if ($string) {
-		$string =~ s/\.+$//;
-		$self->{"mgmname"} = $string;
+		$self->{"mgmname"} = Net::DNS::stripdot($string);
 	}
 
 	return bless $self, $class;
@@ -47,11 +49,18 @@ sub rr_rdata {
 }
 
 
+sub _normalize_dnames {
+	my $self=shift;
+	$self->_normalize_ownername();
+	$self->{'mgmname'}=Net::DNS::stripdot($self->{'mgmname'}) if defined $self->{'mgmname'};
+}
+
+
 sub _canonicalRdata {
     my $self=shift;
     my $rdata = "";
     if (exists $self->{"mgmname"}) {
-		$rdata .= $self->_name2wire($self->{"mgmname"});
+		$rdata .= $self->_name2wire(lc($self->{"mgmname"}));
 	}
 	return $rdata;
 }
@@ -82,7 +91,7 @@ Returns the RR's mailbox field.
 
 =head1 COPYRIGHT
 
-Copyright (c) 1997-2002 Michael Fuhr. 
+Copyright (c) 1997-2002 Michael Fuhr.
 
 Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
